/* dual.c -- bit fields and bitwise operators */
#include <stdio.h>
/* BIT-FIELD CONSTANTS */
/* opaque and show */
#define YES     1
#define NO      0
/* line styles     */
#define SOLID   0
#define DOTTED  1
#define DASHED  2
/* primary colors  */
#define BLUE    4
#define GREEN   2
#define RED     1
/* mixed colors    */
#define BLACK   0
#define YELLOW  (RED | GREEN)
#define MAGENTA (RED | BLUE)
#define CYAN    (GREEN | BLUE)
#define WHITE   (RED | GREEN | BLUE)

/* BITWISE CONSTANTS   */
#define OPAQUE          1
#define FILL_BLUE       8
#define FILL_GREEN      4
#define FILL_RED        2
#define FILL_MASK       14
#define BORDER          256
#define BORDER_BLUE     2048
#define BORDER_GREEN    1024
#define BORDER_RED      512
#define BORDER_MASK     3584
#define B_DOTTED        4096
#define B_DASHED        9192
#define STYLE_MASK     13288

struct box_props {
    unsigned int opaque         : 1;
    unsigned int fill_color     : 3;
    unsigned int                : 4;
    unsigned int show_border    : 1;
    unsigned int border_color   : 3;
    unsigned int border_style   : 2;
    unsigned int                : 2;
 };
const char * colors[8] = {"black", "red", "green", "yellow",
            "blue", "magenta", "cyan", "white"};
char * itobs(int n, char * ps); /* display short as binary string */
int main(void)
{
    union Views     /* look at data as struct or as unsigned short */
    {
        struct box_props st_view;
        unsigned int     sh_view;
    };
    /* create Views object, initialize struct box view */
    union Views box = {{YES, YELLOW , YES, GREEN, DASHED}};
    char bin_str[8 * sizeof(unsigned int) + 1];

    printf("Box is %s.\n",
        box.st_view.opaque == YES? "opaque": "transparent");
    printf ("The border style is ");
    switch(box.st_view.border_style)
    {
        case SOLID  : printf("solid.\n"); break;
        case DOTTED : printf("dotted.\n"); break;
        case DASHED : printf("dashed.\n"); break;
        default     : printf("unknown type.\n");
    }
    printf("The fill color is %s.\n",
        colors[box.st_view.fill_color]);
    printf("The border color is %s.\n",
        colors[box.st_view.border_color]);
    printf("bits are %s\n",
        itobs(box.sh_view,bin_str));
    box.sh_view &= ~FILL_MASK;          /* clear fill bits */
    box.sh_view |= (FILL_BLUE | FILL_GREEN); /* reset fill */
    box.sh_view ^= OPAQUE;               /* toggle opacity */
    printf("After changes, box is %s.\n",
        box.st_view.opaque == YES? "opaque": "transparent");
    printf("The fill color is %s.\n",
        colors[box.st_view.fill_color]);
    printf("The border color is %s.\n",
        colors[(box.sh_view >> 9) & 07]);
    printf("bits are %s\n",
       itobs(box.sh_view,bin_str));
   return 0;
}

/* convert int to binary string */
char * itobs(int n, char * ps)
{
   int i;
   static int size = 8 * sizeof(unsigned int);

   for (i = size - 1; i >= 0; i--, n >>= 1)
        ps[i] = (01 & n) + '0';
   ps[size] = '\0';
   return ps; 
}
